local lanes, frog, grid, d3d = {}, {}, 50

local SAFETY, CAR, LOG = 0, 1, 2



class "Rectangle"



function Rectangle:__init(x, y, w, h)

	self.x = x

	self.w = w

	self.y = y

	self.h = h

end



function Rectangle:Intersects(other)

	return not (self.x >= other.x + other.w 

		or self.x + self.w <= other.x

		or self.y >= other.y + other.h

		or self.y + self.h <= other.y)

end



class("Frog", Rectangle)



function Frog:__init(x, y, w)

	self:__super(x, y, w, w)

end



function Frog:Show()

	d3d:Fill(0xff77ff77);

	d3d:DrawRect(self.x + 1, self.y + 1, self.w, self.w)

end



function Frog:Move(x, y)

	self.x = self.x + x * grid

	self.y = self.y + y * grid

end



function Frog:Update()

	self.x = math.max(0, math.min(d3d.width - grid, self.x + (self.log and self.log.speed or 0)))

	self.y = math.max(0, math.min(d3d.height - grid, self.y))

end



function Frog:Attach(log)

	self.log = log

end





class("Obstacle", Rectangle)



function Obstacle:__init(x, y, w, h, s)

	self:__super(x, y, w, h)

	self.speed = s or 1

end



function Obstacle:Show()

	d3d:Fill(0xff777777);

	d3d:DrawRect(self.x + 1, self.y + 1, self.w - 2, self.h - 2)

end



function Obstacle:Update()

	self.x = self.x + self.speed

	if self.speed > 0 and self.x > d3d.width + grid then

		self.x = - self.w - grid

	elseif self.speed < 0 and self.x < - grid - self.w then

		self.x = d3d.width + grid + self.w

	end

end



class("Lane", Rectangle)



function Lane:__init(y, _type, n, width, spacing, speed)

	y = y *  grid

	self:__super(0, y, d3d.width, grid)

	self.type = _type

	if not width then

		self.color = n

	else

		self.obstacles = {}

		offset = math.random() * 200

		for i = 1, n do

			self.obstacles[#self.obstacles + 1] = Obstacle(offset + i * spacing, y, width * grid, grid, speed)

		end

	end

end



function Lane:Run()

	if self.color then

		d3d:NoStroke()

		d3d:Fill(self.color)

		d3d:DrawRect(self.x, self.y, self.w, self.h)

	end

	if self.obstacles then

		for _, o in pairs(self.obstacles) do

			o:Show()

			o:Update()

		end

	end

end



function Lane:Check(frog)

	if self.type == CAR then

		for _, o in pairs(self.obstacles) do

			if frog:Intersects(o) then

				return true

			end

		end

	elseif self.type == LOG then

		for _, o in pairs(self.obstacles) do

			if frog:Intersects(o) then

				frog:Attach(o)

				return false

			end

		end

		return true

	end

end



function ResetGame()

	frog = Frog(d3d.width/2 - grid/2, d3d.height - grid, grid - 2)

end



function OnSetup()

	d3d = InitD3D(600, 550)

	FrameRate(60)

	ResetGame()

	lanes[0] = Lane(0, SAFETY, 0xff373737)

	lanes[1] = Lane(1, LOG, 3, 1, 150, 3)

	lanes[2] = Lane(2, LOG, 2, 3, 350, -2.5)

	lanes[3] = Lane(3, LOG, 4, 1, 200, 1)

	lanes[4] = Lane(4, LOG, 3, 2, 150, -1.7)

	lanes[5] = Lane(5, SAFETY, 0xff373737)

	lanes[6] = Lane(6, CAR, 3, 1, 150, 2.4)

	lanes[7] = Lane(7, CAR, 2, 2, 150, -3.6)

	lanes[8] = Lane(8, CAR, 1, 3, 150, 2.3)

	lanes[9] = Lane(9, CAR, 4, 1, 150, -1)

	lanes[10] = Lane(10, SAFETY, 0xff373737)

end


function OnFrame()

	d3d:ClearScene(0xff000000)
	d3d:BeginScene()

	d3d:Stroke(0xffffffff)

	frog:Attach(nil)

	for _, lane in pairs(lanes) do

		lane:Run()

		if lane.y == frog.y then

			if lane:Check(frog) then

				ResetGame()

			end

		end

	end

	frog:Update()

	frog:Show()
	d3d:EndScene()
	d3d:Present()
end


local VK_LEFT, VK_UP, VK_RIGHT, VK_DOWN = 0x25, 0x26, 0x27, 0x28

function OnKeyDown(key)

	if key == VK_LEFT then

		frog:Move(-1, 0)

	elseif key == VK_UP then

		frog:Move(0, -1)

	elseif key == VK_RIGHT then

		frog:Move(1, 0)

	elseif key == VK_DOWN then

		frog:Move(0, 1)

	end

end



