local d3d, grid

local col, row, size = 20, 20, 50

local gameOver = false



class("Cell")



function Cell:__init(x, y)

	self.mine = math.random() < .15

	self.x = x - 1

	self.y = y - 1

end



function Cell:Show()

	if self.revealed then

		d3d:Fill(0xffffffff)

		d3d:DrawRect(self.x * size, self.y * size, size, size)

		if self.mine then

			d3d:Fill(0xffff0000)

			d3d:DrawCircle(self.x * size + size * .5, self.y * size + size * .5, size * .25)

		elseif self.revealed > 0 then

			d3d:DrawString(tostring(self.revealed), self.x * size + size * .5 - 12, self.y * size, 0xff000000)

		end

	else

		d3d:Fill(0xffdfdfdf)

		d3d:DrawRect(self.x * size, self.y * size, size, size)

		if self.flag then

			d3d:NoFill()

			d3d:DrawCircle(self.x * size + size * .5, self.y * size + size * .5, size * .25)

			d3d:DrawString(tostring("?"), self.x * size + size * .5 - 12, self.y * size, 0xff000000)

		end

	end

end



function Cell:Reveal()

	if self.flag then return end

	self.revealed = self:CountNeighbours()

	if self.revealed == -1 then

		gameOver = true

	end

	if gameOver or self.revealed == 0 then

		self:RevealNeighbours()

	end

end



function Cell:Flag()

	self.flag = not self.flag

end



function Cell:CountNeighbours()

	if self.mine then return -1 end

	local count = 0

	for i = self.x, self.x + 2 do

		for j = self.y, self.y + 2 do

			if grid[i] and grid[i][j] and grid[i][j].mine then

				count = count + 1

			end

		end

	end

	return count

end



function Cell:RevealNeighbours()

	for i = self.x, self.x + 2 do

		for j = self.y, self.y + 2 do

			if grid[i] and grid[i][j] and not grid[i][j].revealed then

				grid[i][j]:Reveal()

			end

		end

	end

end



function RestartGame()

	grid = {}

	for i = 1, col do

		grid[i] = {}

		for j = 1, row do

			grid[i][j] = Cell(i, j)

		end

	end

	gameOver = false

end



function OnSetup()

	d3d = InitD3D(col * size, row * size)

	d3d:Stroke(0xffffffff)

	RestartGame()

end



function OnFrame()

	d3d:ClearScene(0xff000000)

	d3d:BeginScene()

	for i = 1, col do

		for j = 1, row do

			grid[i][j]:Show()

		end

	end

	if gameOver then

		local x, y = 20, d3d.height * .5

		for i = -10, 10 do

			for j = -10, 10 do

				d3d:DrawString("Game Over! Press Space.", x + i, y + j, 0xffffffff)

			end

		end

		d3d:DrawString("Game Over! Press Space.", x, y, 0xff000000)

	end

	d3d:EndScene()

	d3d:Present()

end



function OnKeyDown(k)

	if gameOver and k == 0x20 then

		RestartGame()

	end

end



function OnMouseLClick(x, y)

	if gameOver then return end

	grid[math.ceil(x / size)][math.ceil(y / size)]:Reveal()

end



function OnMouseRClick(x, y)

	if gameOver then return end

	grid[math.ceil(x / size)][math.ceil(y / size)]:Flag()

end