local d3d, lives, player, obstacles, enemies, pprojectiles, eprojectiles, enemyspeed



class("Object")



function Object:__init(x, y, w, h, xspeed, yspeed)

	self.x = x

	self.y = y

	self.w = w

	self.h = h or w

	self.xspeed = xspeed or 0

	self.yspeed = yspeed or 0

end



function Object:Move(dx, dy)

	self.x = self.x + dx

	self.y = self.y + dy

end



function Object:Update()

	self.x = self.x + self.xspeed

	self.y = self.y + self.yspeed

end



function Object:Collides(o)

	return not (self.x >= o.x + o.w 

		or self.x + self.w <= o.x

		or self.y >= o.y + o.h

		or self.y + self.h <= o.y)

end



function Object:Show()

	d3d:Fill(0xffffffff)

	d3d:DrawRect(self.x, self.y, self.w, self.h)

end



class("Enemy", Object)

function Enemy:Show()

	d3d:Fill(0xffffffff)

	d3d:DrawRect(self.x, self.y, self.w, self.h)

end



function Enemy:Fire()

	if not self.last_fire or  self.last_fire < os.clock() then

		self.last_fire = os.clock() + 2

		eprojectiles[#eprojectiles + 1] = Object(self.x + 20, self.y + 10, 3, 20, 0, 10)

	end

end



function Enemy:Update()

	self.x = self.x + enemyspeed

	if self.x < 100 or self.x > d3d.width - 100 then

		return true

	end

end



class("Obstacle", Object)

function Obstacle:__init(...)

	self:__super(...)

	self.state = 255

end



function Obstacle:Show()

	d3d:Fill((0xff << 24) + ((255 - self.state) << 16) + (self.state << 8))

	d3d:DrawRect(self.x, self.y, self.w, self.w)

end



class("Player", Object)

function Player:Show()

	d3d:Fill(0xffffffff)

	d3d:DrawRect(self.x, self.y, self.w, self.w)

end



function Player:Fire()

	if not self.last_fire or  self.last_fire < os.clock() then

		self.last_fire = os.clock() + .5

		pprojectiles[#pprojectiles + 1] = Object(self.x + 14, self.y - 10, 3, 20, 0, -10)

	end

end

 -- (x, y, w, h, xspeed, yspeed)



function RestartGame()

	lives = 3

	enemyspeed = .25

	player = Player(d3d.width / 2 - 15, d3d.height - 80, 30)

	obstacles = {

		Obstacle(d3d.width / 2 - 220, d3d.height - 180, 80),

		Obstacle(d3d.width / 2 - 40, d3d.height - 180, 80),

		Obstacle(d3d.width / 2 + 140, d3d.height - 180, 80),

	}

	pprojectiles, eprojectiles = {}, {}

	enemies = {}

	for i = 1, 6 do

		for j = 1, 6 do

			enemies[#enemies + 1] = Enemy(50 + 70 * i, 50 + 50 * j, 40, 20, 1)

		end

	end

end



function OnSetup()

	d3d = InitD3D(800, 600)

	RestartGame()

	FrameRate(30)

end



function OnFrame()

	d3d:ClearScene(0xff000000)

	d3d:BeginScene()

	for _, o in pairs(obstacles) do

		if o.state > 0 then

			o:Show()

		end

	end

	for _, o in pairs(pprojectiles) do

		if not o.deleted then

			o:Update()

			o:Show()

			for _, e in pairs(obstacles) do

				if e.state > 0 and o:Collides(e) then

					o.deleted, e.state = true, e.state - 8

				end

			end

			for _, e in pairs(enemies) do

				if not e.deleted and o:Collides(e) then

					o.deleted, e.deleted = true, true

				end

			end

		end

	end

	for _, o in pairs(eprojectiles) do

		if not o.deleted then

			o:Update()

			o:Show()

			for _, e in pairs(obstacles) do

				if e.state > 0 and o:Collides(e) then

					o.deleted, e.state = true, e.state - 2

				end

			end

			if player:Collides(o) then

				o.deleted, lives = true, lives - 1

				if lives < 0 then

					RestartGame()

				end

			end

		end

	end

	local moveDown = false

	for _, o in pairs(enemies) do

		if not o.deleted then

			moveDown = moveDown or o:Update()

			if math.random() < 0.001 then

				o:Fire()

			end

			o:Show()

			for _, e in pairs(obstacles) do

				if e.state > 0 and o:Collides(e) then

					o.deleted, e.state = true, e.state - 20

				end

			end

			if player:Collides(o) then

				o.deleted, lives = true, lives - 1

				if lives < 0 then

					RestartGame()

				end

			end

		end

	end

	if moveDown then

		enemyspeed = enemyspeed < 0 and -enemyspeed+.15 or -enemyspeed-.15

		for _, e in pairs(enemies) do

			e.y = e.y + 5

		end

	end

	player:Show()

	d3d:Fill(0xff00ff00)

	for i = 1, lives do

		d3d:DrawRect(10 + i * 15, d3d.height - 10, 10, 10)

	end

	d3d:EndScene()

	d3d:Present()

end



local VK_SPACE, VK_LEFT, VK_UP, VK_RIGHT = 0x20, 0x25, 0x26, 0x27

function OnKeyDown(k)

	if k == VK_SPACE or k == VK_UP then

		player:Fire()

	end

	if k == VK_LEFT then

		player:Move(-10, 0)

	end

	if k == VK_RIGHT then

		player:Move(10, 0)

	end

end